package org.zeroturnaround.javarebel.integration.servlet.oc4j;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.bundled.javassist.expr.ExprEditor;
import org.zeroturnaround.bundled.javassist.expr.MethodCall;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.RebelServletContext;
import org.zeroturnaround.javarebel.integration.generic.BaseServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.servlet.JavaRebelResourceServlet;

/**
 * Transforms <code>com.evermind.server.http.HttpApplication</code>
 * by registering OC4J Servlet Context with JavaRebel and overriding all resource providing methods.
 * <p>
 * Also {@link JavaRebelResourceServlet} will be registered.
 * 
 * @author Rein Raudjärv
 */
public class HttpApplicationCBP extends BaseServletContextImplClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    // Override resource methods
    super.process(cp, cl, ctClass);
    
    // Register Servlet Context
    try {
      CtMethod m = ctClass.getDeclaredMethod("initDynamic");
      m.insertBefore(
          "org.zeroturnaround.javarebel.ServletIntegrationFactory.getInstance()" +
          "  .registerServletContext((" + RebelServletContext.class.getName() + ") $0);");
    } catch (NotFoundException e) {
    }

    try {
      registerRebelServlet(cp, cl, ctClass);
    } catch (CannotCompileException e) {
      LoggerFactory.getInstance().errorEcho(e);
    }
  }
  
  private void registerRebelServlet(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    final boolean verbose = LoggerFactory.getInstance().isEnabled();
    
    final String shouldDefineRebelServlet = "__shouldDefineRebelServlet";
    final String newRebelServletDescriptor = "__newRebelServletDescriptor";
    
    final String defaultServletPattern = "/";
    final String servletClass = JavaRebelResourceServlet.class.getName();
    final String servletName = servletClass;
    
    cp.importPackage("java.util");
    cp.importPackage("java.net");
    cp.importPackage("com.evermind.util");
    cp.importPackage("com.evermind.server.http");
    cp.importPackage("com.evermind.server.http.deployment");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.servlet.oc4j");
    
    ctClass.addMethod(CtNewMethod.make(
        "private boolean " + shouldDefineRebelServlet + "() {" +
        "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
        "  if (!integration.hasReplacedResources((RebelServletContext) $0))" +
        "    return false;" +

        "  List mappings = this.config.getServletMappings();" +
        "  Iterator it = mappings.iterator();" +
        "  while (it.hasNext()) {" +
        "    ServletMapping mapping = (ServletMapping) it.next();" +
        "    if (\"" + defaultServletPattern + "\".equals(mapping.getURLPattern()))" +
        "      return false;" +
        "  }" +
        "  return true;" +
        "}",
        ctClass));
    
    ctClass.addMethod(CtNewMethod.make(
        "private ServletDescriptor " + newRebelServletDescriptor + "() {" +
        "    ServletDescriptor sd = new ServletDescriptor();" +
        "    ((ServletDescriptorExtra) sd).__setParent(this.config);" +
        "    sd.setName(\"" + servletName + "\");" +
        "    sd.setClassName(\"" + servletClass + "\");" +
        (verbose ? "    LoggerFactory.getInstance().log(\"New servlet descriptor: \" + sd);" : "") +
        "  return sd;" +
        "}",
        ctClass));
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("initPreloadServlets");
      m.instrument(new ExprEditor() {
        public void edit(MethodCall m) throws CannotCompileException {
          if (m.getClassName().equals("com.evermind.server.http.HttpApplicationConfig")
              && m.getMethodName().equals("getServlets"))
            m.replace("{" +
                "  List descriptors = $proceed($$);" +
                "  try {" +
                (verbose ? "    LoggerFactory.getInstance().log(\"Servlet descriptors: \" + descriptors);" : "") +

                "    if (" + shouldDefineRebelServlet + "()) {" +
                "      ServletDescriptor sd = " + newRebelServletDescriptor + "();" +

                "      List newDescriptors = new ArrayList();" +
                "      newDescriptors.addAll(descriptors);" +
                "      newDescriptors.add(sd);" +
                "      descriptors = newDescriptors;" +
                "    }" +
                "  } catch (Throwable t) {" +
                "    LoggerFactory.getInstance().error(t);" +
                "  }" +
                "  $_ = descriptors;" +
            "}");
        }
      });
    } catch (NotFoundException e) {
    }
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("loadServlet");
      m.instrument(new ExprEditor() {
        public void edit(MethodCall m) throws CannotCompileException {
          if (m.getClassName().equals("com.evermind.server.http.HttpApplicationConfig")
              && m.getMethodName().equals("getServletDescriptor"))
            m.replace("{" +
                "  ServletDescriptor descriptor = $proceed($$);" +
                "  try {" +
                (verbose ? "    LoggerFactory.getInstance().log(\"Servlet descriptor: \" + descriptor + \"(\" + $1 + \")\");" : "") +
                "    if (" + shouldDefineRebelServlet + "() && \"" + servletName + "\".equals($1))" +
                "      descriptor = " + newRebelServletDescriptor + "();" +
                "  } catch (Throwable t) {" +
                "    LoggerFactory.getInstance().error(t);" +
                "  }" +
                "  $_ = descriptor;" +
            "}");
        }
      });
    } catch (NotFoundException e) {
    }
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("initServletMappings");
      m.instrument(new ExprEditor() {
        public void edit(MethodCall m) throws CannotCompileException {
          if (m.getClassName().equals("com.evermind.server.http.HttpApplicationConfig")
              && m.getMethodName().equals("getServletMappings"))
            m.replace("{" +
                "  List mappings = $proceed($$);" +
                "  try {" +
                (verbose ? "    LoggerFactory.getInstance().log(\"Servlet mappings: \" + mappings);" : "") +
                (verbose ? "    LoggerFactory.getInstance().log(\"Should define rebel servlet: \" + " + shouldDefineRebelServlet + "());" : "") +

                "    if (" + shouldDefineRebelServlet + "()) {" +
                "      ServletMapping sm = new ServletMapping();" +
                "      sm.setServletName(\"" + servletName + "\");" +
                "      ((ServletMappingExtra) sm).__setURLPattern(\"" + defaultServletPattern + "\");" +
                (verbose ? "      LoggerFactory.getInstance().log(\"New servlet mapping: \" + sm);" : "") +

                "      List newMappings = new ArrayList();" +
                "      newMappings.addAll(mappings);" +
                "      newMappings.add(sm);" +
                "      mappings = newMappings;" +
                "    }" +
                "  } catch (Throwable t) {" +
                "    LoggerFactory.getInstance().error(t);" +
                "  }" +
                "  $_ = mappings;" +
            "}");
        }
      });
    } catch (NotFoundException e) {
    }
  }
}
