package org.zeroturnaround.javarebel.integration.servlet.resin;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>com.caucho.server.webapp.Application</code>
 * by registering Resin servlet context instance with JavaRebel.
 * <p>
 * Since Resin 3.1.x the transformed class was renamed as
 * <code>com.caucho.server.webapp.WebApp</code>.
 * As JavaRebel transforms and writes that class into a special JAR
 * the transformation code is contained in the <code>javarebel-integration</code> plugin.
 * 
 * @author Toomas Römer
 */
public class ResinApplicationCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    ctClass.getMethod("init", "()V").insertAfter(
        "ServletIntegrationFactory.getInstance().registerServletContext((RebelServletContext) $0);");
    
    ctClass.getMethod("getRealPath", "(Ljava/lang/String;)Ljava/lang/String;").insertBefore(
        "{" + 
        "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
        "  RebelServletContext sc = (RebelServletContext) $0;" +
        "  if (integration.isResourceReplaced(sc, $1))" +
        "    return integration.getRealPath(sc, $1);" + 
        "}");
  }
}
