package org.zeroturnaround.javarebel.integration.servlet.util;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URLEncoder;
import java.util.StringTokenizer;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

/**
 * @author Rein Raudjärv
 */
public class ServletUtil {

  /**
   * Returns the requested resource URI.
   * <p>
   * The requested resource may also be the target of a <code>RequestDispatcher.include()</code>
   * 
   * @param req request.
   * @return requested resource URI.
   */
  public static String getUriFromRequest(HttpServletRequest req) {
    /*
     * Check to see if the requested JSP has been the target of a
     * RequestDispatcher.include()
     */
    String uri = (String) req.getAttribute("javax.servlet.include.servlet_path");
    if (uri != null) {
      /*
       * Requested JSP has been target of RequestDispatcher.include(). Its
       * path is assembled from the relevant javax.servlet.include.* request
       * attributes
       */
      String pathInfo = (String) req.getAttribute("javax.servlet.include.path_info");
      if (pathInfo != null) {
        uri += pathInfo;
      }
    }
    else {
      /*
       * Requested JSP has not been the target of a
       * RequestDispatcher.include(). Reconstruct its path from the request's
       * getServletPath() and getPathInfo()
       */
      uri = req.getServletPath();
      String pathInfo = req.getPathInfo();
      if (pathInfo != null) {
        uri += pathInfo;
      }
    }
    return uri;
  }

  /**
   * Returns <code>true</code> if the servlet context has a resource with the given path.
   */
  public static boolean exists(ServletContext servletContext, String path) {
    try {
      return servletContext.getResource(path) != null;
    } catch (MalformedURLException e) {
      return false;
    }
  }

  /**
   * Returns URI of the parent directory. 
   * 
   * @param uri URI of a directory.
   */
  public static String getParentDirectory(String uri) {
    if (uri == null)
      return uri;
    
    if (uri.endsWith("/"))
      uri = uri.substring(0, uri.length() - 1);

    return uri.substring(0, uri.lastIndexOf("/") + 1);
  }

  /**
   * Encodes URL.
   */
  public static String encodeUrl(String url) {
    if (url == null)
      return null;
    
    try {
      StringBuffer sb = new StringBuffer();
      StringTokenizer st = new StringTokenizer(url, "/", true);
      while (st.hasMoreTokens()) {
        String token = st.nextToken();
        if ("/".equals(token)) {
          sb.append(token);
        } else {
          sb.append(URLEncoder.encode(token, "UTF-8"));
        }
      }
      return sb.toString();
    } catch (UnsupportedEncodingException e) {
      throw new RuntimeException(e);
    }
  }

}
